require('dotenv').config()
const soap = require('soap');

const wsdlurl = 'https://partner-api.sihot.com/PDOCS/API/CBS/SihotServices01?wsdl';

let client = null;

async function init() {
    try {
        console.log("init SOAP Client")
        client = await soap.createClientAsync(wsdlurl);
    }
    catch (e) {
        console.error("Initialization of SOAP Client failed.")
        console.error(e)
    }
}

//--------------------------
/*
require('dotenv').config()
const soap = require('soap');

const localwsdl = "./SihotServices01.xml";
const wsEndPoint = "https://partner-api.sihot.com/PDOCS/API/CBS/SihotServices01";

let securityID = ""
let tokenValidUntil = new Date()
let client = null;

async function init() {
    try {
        console.log("init SOAP Client")

        client = await soap.createClientAsync(localwsdl);
        client.setEndpoint(wsEndPoint) //- overwrite the SOAP service endpoint address

    }
    catch (e) {
        console.error("Initialization of SOAP Client failed.")
        console.error(e)
    }
}
*/

// build the AuenticateRequest
const AuthenticateRequest = {
    AuthenticationInfos: {
        "user": process.env.SIHOT_USER,
        "password": process.env.SIHOT_PASSWORD,
        "hotel": process.env.SIHOT_HOTEL
    }
};

// local variabels to store the securityID as well as the invalidation timestamp
let securityID = ""
let tokenValidUntil = new Date()
async function getSecurityID() {
    try {
        const AuthenticateResponse = await client.AuthenticateAsync(AuthenticateRequest);

        securityID = AuthenticateResponse[0].Authentication[0].SecurityID;
        tokenValidUntil = new Date();
        // calculate the datetime when the token will be invalidated
        tokenValidUntil.setTime(tokenValidUntil.getTime() + (parseInt(AuthenticateResponse[0].Authentication[0].DurationInSec) * 1000))
        console.log("New security received: " + securityID)
        console.log("New security valid until: " + tokenValidUntil.toLocaleString())
    }
    catch (e) {
        console.error("Authentification Failed");
        console.error(e)
    }
}

let callingDate = new Date().toISOString().substring(0, 10)
async function getInhouseGuests() {
    // ensure the securityID is still valid
    const now = new Date()
    if (tokenValidUntil <= now) {
        console.log(tokenValidUntil + " now it was " + now)
        console.log("getting new securityID .. ")
        await getSecurityID();
    }

    const S_GUEST_IN_HOUSE_SEARCH_V001Request = {
        Authentication: {
            SecurityID: securityID
        },
        GuestInHouseSearch: {
            date: callingDate
        }
    }

    const S_GUEST_IN_HOUSE_SEARCH_V001Response = await client.S_GUEST_IN_HOUSE_SEARCH_V001Async(S_GUEST_IN_HOUSE_SEARCH_V001Request);
    if (S_GUEST_IN_HOUSE_SEARCH_V001Response.length <= 0 )
    {
        console.error ("Response invalid")
    }

    if ( S_GUEST_IN_HOUSE_SEARCH_V001Response[0].Result.Success !== "true" )
    {
        console.error ("Method call failed")
        console.error (S_GUEST_IN_HOUSE_SEARCH_V001Response[0].Result.ErrorMsg)
        console.error (S_GUEST_IN_HOUSE_SEARCH_V001Response[0].Result['MSG-LIST'])
    }
    showGuests(S_GUEST_IN_HOUSE_SEARCH_V001Response)
    return S_GUEST_IN_HOUSE_SEARCH_V001Response;
}

function showGuests(S_GUEST_IN_HOUSE_SEARCH_V001Response) {
    if ( S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room == undefined )
    {
        console.log ("Roomlist was empty")
        return
    }
    
    for ( let i = 0; i < S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room.length; i++ )
    {
        const cat = S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room[i].category
        const room = S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room[i].roomState[0].room
        let sharerNames = ""
        for (j = 0; j < S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room[i].Person.length; j++) {
            if (sharerNames.length > 0) {
                sharerNames += ","
            }
            sharerNames += S_GUEST_IN_HOUSE_SEARCH_V001Response[0].room[i].Person[j].name1
        }
        console.log(`Room ${room} of type ${cat} occupied by: ${sharerNames} `);
    }
}

async function main() {
    await init()
    await getSecurityID()

    const today = new Date().toISOString().substring(0, 10)
    const response = await getInhouseGuests(today);
    setInterval ( () => {getInhouseGuests (today)}, 1820000)    
}

main()